function results = ComputeExcessRate(eventTimesVectors,responses,binWidth,neutralRate,stimDuration,startTime)
% function results = ComputeExcessRate(eventTimesVectors,responses,binWidth,neutralRate,stimDuration,startTime)
%
% Computes excess rate timecourse given stimulus vectors specifying the
% timing of events during the trial duration, as well as a vector of the
% subject's responses.
%
% John Sheppard, 29-Jul-2013
% Modified from code by Anne Churchland (2010-2011).

if ~exist('binWidth','var');
    binWidth = 333; % units of milliseconds
end

if ~exist('neutralRate','var');
    neutralRate = 10.5; % units of events/second
end

if ~exist('stimDuration','var');
    stimDuration = 1000; % units of milliseconds
end

if numel(eventTimesVectors) ~= numel(responses)
    error('Number of event vectors must match the number of trial responses!');
end

% First, construct an array containing 1's indicating individual event
% arrivals. Each row represents a trial. Each column represents a 1-ms time bin.
numTrials = numel(responses);
eventsMatrix = uint8(zeros(numTrials, stimDuration));
h = waitbar(0,'Going through the trials') ;
for trialCount = 1:length(eventTimesVectors)
  h = waitbar(trialCount/length(eventTimesVectors));
    thisTrialEventTimes = eventTimesVectors{trialCount};
    eventsMatrix(trialCount,thisTrialEventTimes) = 1;
end
close(h);

% Function will identify trials that have a number of trials corresponding
% to the "neutral evidence" event rate falling outside the sliding window
% in which "excess rate" is computed. This quantity is referred to as
% "neutral event count."
neutralEventCount = round(neutralRate*(stimDuration-binWidth)/stimDuration);

resolution = 10; % Compute excess rate for a sliding window with resolution of 10 ms (between sliding windows)

% Initialize variables
timeVector = [];
leftChoiceMeanWindowEventCounts = [];
rightChoiceMeanWindowEventCounts = [];
excessRateEsts = [];
excessRateStdErrs = [];
shuffledExcessRateEsts = [];
shuffledExcessRateStdErrs = [];

h = waitbar(0,'Going through the trials') ;
resolution = 10;

binCount = 0;

% Go through each position of the sliding window.
for windowStartPosition = startTime:resolution:(stimDuration-binWidth) 
    
    binCount = binCount+1;
    
    % Display the overall progress of code execution to user.
    h = waitbar(windowStartPosition/((stimDuration-(binWidth))));
    
    % Compute the center of the sliding window.
    timeVector = [timeVector; (windowStartPosition + (windowStartPosition + binWidth - 1))/2 ];
    
    % Compute the # of events outside the window of interest for all trials
    % in input dataset.
    numEventsOutsideWindow = sum(eventsMatrix(:,windowStartPosition+binWidth:stimDuration),2) + sum(eventsMatrix(:,1:windowStartPosition-1),2);
    
    % These are the trials that have the specificed # of events falling
    % outside the current window (i.e., neutral event rate outside window).
    trialsMatchSpec = find(numEventsOutsideWindow == neutralEventCount);
    
    % Get all the choices for the trials of interest.
    choiceVector = responses(trialsMatchSpec);
    
    % Get the # events in sliding window for all trials of interest.
    thisWindowEventCounts = sum(eventsMatrix(trialsMatchSpec,windowStartPosition:windowStartPosition+binWidth-1),2);
    
    % Compute the average # of events in sliding window
    % for trials preceding low-rate (left) and high-rate (right) choices.
    leftChoiceTrialWindowEventCounts = thisWindowEventCounts(choiceVector == 1);
    rightChoiceTrialWindowEventCounts = thisWindowEventCounts(choiceVector == 2);
    leftChoiceMeanWindowEventCounts = [leftChoiceMeanWindowEventCounts, mean(leftChoiceTrialWindowEventCounts) ];
    rightChoiceMeanWindowEventCounts =  [rightChoiceMeanWindowEventCounts, mean(rightChoiceTrialWindowEventCounts) ];
    
    % Note: The quantity computed below is not truly "excess rate" because the units 
    % are raw event counts. Later on this quantity will be normalized for the sliding
    % window size so that the units are events/second.
    excessRateEsts = [excessRateEsts, mean(rightChoiceTrialWindowEventCounts) - mean(leftChoiceTrialWindowEventCounts)];
    
    % Get a standard error estimate on the (un-normalized) excess rate via bootstrapping.
    numBootSamples = 1000;
    leftChoiceTrialBootSample = uint16(unidrnd(length(leftChoiceTrialWindowEventCounts),length(leftChoiceTrialWindowEventCounts),numBootSamples));
    rightChoiceTrialBootSample = uint16(unidrnd(length(rightChoiceTrialWindowEventCounts),length(rightChoiceTrialWindowEventCounts),numBootSamples));
    excessRateBootstraps = mean(rightChoiceTrialWindowEventCounts(rightChoiceTrialBootSample)) - mean(leftChoiceTrialWindowEventCounts(leftChoiceTrialBootSample));
    thisWindowExcessRateStdErr = std(excessRateBootstraps);
    excessRateStdErrs = [excessRateStdErrs; thisWindowExcessRateStdErr];
    
    % Compare the observed differences in window event rates (i.e., excess
    % rate) between left and right choices with a "shuffle" generated by
    % randomly assigning subject responses to every trial of interest
    % by random sampling from the observed response distribution regardless of
    % the subject's actual response on that trial.
    randTrials = randperm(length(choiceVector));
    randChoiceVector = choiceVector(randTrials);
    shuffledLeftChoiceTrialWindowEventCounts = thisWindowEventCounts(randChoiceVector == 1);
    shuffledRightChoiceTrialWindowEventCounts = thisWindowEventCounts(randChoiceVector == 2);
    shuffledExcessRateEsts = [shuffledExcessRateEsts, mean(shuffledRightChoiceTrialWindowEventCounts)-mean(shuffledLeftChoiceTrialWindowEventCounts)];
    
    % Bootstrap analysis can also be performed on the shuffled data to get a
    % standard error estimate, although these standard errors were not plotted 
    % in the paper (Figs. 5-6).
    shuffledLeftChoiceTrialBootSample = uint16(unidrnd(length(shuffledLeftChoiceTrialWindowEventCounts),length(shuffledLeftChoiceTrialWindowEventCounts),numBootSamples));
    shuffledRightChoiceTrialBootSample = uint16(unidrnd(length(shuffledRightChoiceTrialWindowEventCounts),length(shuffledRightChoiceTrialWindowEventCounts),numBootSamples));
    shuffledExcessRateBootstraps = mean(shuffledRightChoiceTrialWindowEventCounts(shuffledRightChoiceTrialBootSample)) - mean(shuffledLeftChoiceTrialWindowEventCounts(shuffledLeftChoiceTrialBootSample));
    thisWindowShuffledExcessRateStdErr = std(shuffledExcessRateBootstraps);
    shuffledExcessRateStdErrs = [shuffledExcessRateStdErrs, thisWindowShuffledExcessRateStdErr];
    
end

close(h);

scaleFactor = 1000/binWidth; % Use this to normalize the data to units of events/second

% Finally, prepare output dataset.
results.stimDuration = stimDuration;
results.binWidth = binWidth;
results.timeVector = timeVector;
% All the quantities below are scaled to units of events/second.
results.leftChoiceMeanWindowRates = scaleFactor*leftChoiceMeanWindowEventCounts;
results.rightChoiceMeanWindowRates = scaleFactor*rightChoiceMeanWindowEventCounts;
results.excessRateEsts = scaleFactor*excessRateEsts;
results.excessRateStdErrs = scaleFactor*excessRateStdErrs;
results.excessRateShuffleEsts = scaleFactor*shuffledExcessRateEsts;
results.excessRateShuffleStdErrs = scaleFactor*shuffledExcessRateStdErrs;

end % EOF
