% John Sheppard, July 24, 2013
% Questions? sheppard@cshl.edu

% This script calls various functions included with this tutorial to compute and 
% store estimates and standard errors of psychometric parameters for the rat and human
% datasets. It has been pre-run so that the "psychFit" data structures
% containing the psychometric measures used in the paper are
% pre-compiled in the tutorial (see data folder). Thus, this script need not be executed 
% to run the plotting scripts for Figures 2-6.

%% Humans

clear

dataFolder = ['../data/'];

humanDatasets = {'human1','human2','human3','human4','human5','human6','human7','humanPooled'};

trialTypeNames = {'audLowRel','audHighRel','visLowRel','visHighRel','audLowRel_visLowRel', ...
    'audHighRel_visLowRel', 'audLowRel_visHighRel','audHighRel_visHighRel'};

for humanCount = 1:numel(humanDatasets)
    
    clear psychFits
    
    thisDataset = humanDatasets{humanCount};
    % Load the dataset.
    % If the index exceeds numHumans, this indicates to analyze the pooled
    % dataset.
    dataPath = [dataFolder, thisDataset '.mat'];
    psychFits.name = thisDataset;
    load(dataPath);
    
    % Analyze single sensory data.
    trialTypeCount = 0;
    
    % Auditory trials
    for audRelLevel = 1:2
        trialTypeCount = trialTypeCount+1;
        % Get trials for current trial type.
        theseTrials = find([dat.audStimPresented] & ~[dat.visStimPresented] & [dat.audRelLevel] == audRelLevel);
        theseEventRates = [dat(theseTrials).audEventRate];
        theseResponses = [dat(theseTrials).response];
        
        % Get psychometric fits.
        [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
        
        % Store psychometric fits in structure
        psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
        psychFits.trialTypes(trialTypeCount).choiceDataByRate = choiceDataByRate;
        psychFits.trialTypes(trialTypeCount).psychParams = psychParams;
        psychFits.trialTypes(trialTypeCount).psychCurve = psychCurve;
        psychFits.trialTypes(trialTypeCount).excessRateResults = ComputeExcessRate({dat(theseTrials).audEventTimes},theseResponses,333,10.5,1000,20);
    end
    
    % Visual trials
    for visRelLevel = 1:2
        trialTypeCount = trialTypeCount+1;
        % Get trials for current trial type.
        theseTrials = find(~[dat.audStimPresented] & [dat.visStimPresented] & [dat.visRelLevel] == visRelLevel);
        theseEventRates = [dat(theseTrials).visEventRate];
        theseResponses = [dat(theseTrials).response];
        
        % Get psychometric fits.
        [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
        
        % Store psychometric fits in structure
        psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
        psychFits.trialTypes(trialTypeCount).choiceDataByRate = choiceDataByRate;
        psychFits.trialTypes(trialTypeCount).psychParams = psychParams;
        psychFits.trialTypes(trialTypeCount).psychCurve = psychCurve;
        psychFits.trialTypes(trialTypeCount).excessRateResults = ComputeExcessRate({dat(theseTrials).visEventTimes},theseResponses,333,10.5,1000,20);
    end
    
    % Analyze multisensory data.
    for visRelLevel = 1:2
        for audRelLevel = 1:2
            trialTypeCount = trialTypeCount+1;
            
            cueConflicts = -2:1:2;
            clear allConflict*
            
            for cueConflictCount = 1:numel(cueConflicts)
                
                cueConflict = cueConflicts(cueConflictCount);
                % Get trials for current trial type.
                theseTrials = find([dat.audStimPresented] & [dat.visStimPresented] & [dat.audRelLevel] == audRelLevel & [dat.visRelLevel] == visRelLevel & [dat.cueConflict] == cueConflict);
                
                % For multisensory trials, the stimulus is defined as the mean event rate
                % averaged between auditory and visual modalities.
                theseEventRates = 0.5*[dat(theseTrials).audEventRate] + 0.5*[dat(theseTrials).visEventRate];
                theseResponses = [dat(theseTrials).response];
                
                % Get psychometric fits.
                [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
                
                % Store psychometric fits in structure
                psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).cueConflict = cueConflict;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).choiceDataByRate = choiceDataByRate;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).psychParams = psychParams;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).psychCurve = psychCurve;
                
                allConflictPSEEsts(cueConflictCount) = psychParams(5).est;
                allConflictPSEStdErrs(cueConflictCount) = psychParams(5).stdErr;
                
            end
            
            [visWeightEst,visWeightStdErr] = ComputeVisualWeight(cueConflicts,allConflictPSEEsts,allConflictPSEStdErrs);
            psychFits.trialTypes(trialTypeCount).visWeightEst = visWeightEst;
            psychFits.trialTypes(trialTypeCount).visWeightStdErr = visWeightStdErr;
            
            % Get single sensory threshold estimates corresponding to this
            % multisensory trial type in order to calculate the predicted visual weight.
            audThresholdEst = psychFits.trialTypes(audRelLevel).psychParams(2).est;
            audThresholdStdErr = psychFits.trialTypes(audRelLevel).psychParams(2).stdErr;
            visThresholdEst = psychFits.trialTypes(2+visRelLevel).psychParams(2).est;
            visThresholdStdErr = psychFits.trialTypes(2+visRelLevel).psychParams(2).stdErr;
            
            [predVisWeightEst,predVisWeightStdErr] = PredictVisualWeight(audThresholdEst,visThresholdEst,audThresholdStdErr,visThresholdStdErr);
            psychFits.trialTypes(trialTypeCount).predVisWeightEst = predVisWeightEst;
            psychFits.trialTypes(trialTypeCount).predVisWeightStdErr = predVisWeightStdErr;
            
        end
    end
    
    outPath = [dataFolder, thisDataset '_psychFits.mat'];
    delete(outPath);
    save(outPath,'psychFits');
    
end


%% Rats

clear

ratDatasets = {'rat1','rat2','rat3','rat3_fig2_data','rat4','rat5','ratPooled'};

dataFolder = ['../data/'];

trialTypeNames = {'audLowRel','audHighRel','vis','audLowRel_vis','audHighRel_vis'};

for ratCount = 1:numel(ratDatasets)
    
    clear psychFits
    
    thisDataset = ratDatasets{ratCount};
    
    % Load the dataset.
    dataPath = [dataFolder, thisDataset '.mat'];
    load(dataPath);
    
    psychFits.name = thisDataset;
    
    % Analyze single sensory data
    trialTypeCount = 0;
    
    % Auditory trials
    for audRelLevel = 1:2
        trialTypeCount = trialTypeCount+1;
        % Get trials for current trial type.
        theseTrials = find([dat.audStimPresented] & ~[dat.visStimPresented] & [dat.audRelLevel] == audRelLevel);
        theseEventRates = [dat(theseTrials).audEventRate];
        theseResponses = [dat(theseTrials).response];
        
        % Get psychometric fits.
        [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
        
        % Store psychometric fits in structure
        psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
        psychFits.trialTypes(trialTypeCount).choiceDataByRate = choiceDataByRate;
        psychFits.trialTypes(trialTypeCount).psychParams = psychParams;
        psychFits.trialTypes(trialTypeCount).psychCurve = psychCurve;
        psychFits.trialTypes(trialTypeCount).excessRateResults = ComputeExcessRate({dat(theseTrials).audEventTimes},theseResponses,280,12.5,1065,1);
    end
    
    % Visual trials
    for visRelLevel = 1
        trialTypeCount = trialTypeCount+1;
        % Get trials for current trial type.
        theseTrials = find(~[dat.audStimPresented] & [dat.visStimPresented] & [dat.visRelLevel] == visRelLevel);
        theseEventRates = [dat(theseTrials).visEventRate];
        theseResponses = [dat(theseTrials).response];
        
        % Get psychometric fits.
        [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
        
        % Store psychometric fits in structure
        psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
        psychFits.trialTypes(trialTypeCount).choiceDataByRate = choiceDataByRate;
        psychFits.trialTypes(trialTypeCount).psychParams = psychParams;
        psychFits.trialTypes(trialTypeCount).psychCurve = psychCurve;
        psychFits.trialTypes(trialTypeCount).excessRateResults = ComputeExcessRate({dat(theseTrials).visEventTimes},theseResponses,280,12.5,1065,1);
    end
    
    % Analyze multisensory data
    for visRelLevel = 1
        for audRelLevel = 1:2
            trialTypeCount = trialTypeCount+1;
            
            % Only cue conflicts of -2, 0 and +2 were presented to rats.
            cueConflicts = [-2 0 2];
            clear allConflict*
            
            for cueConflictCount = 1:numel(cueConflicts)
                
                cueConflict = cueConflicts(cueConflictCount);
                % Get trials for current trial type.
                theseTrials = find([dat.audStimPresented] & [dat.visStimPresented] & [dat.audRelLevel] == audRelLevel & [dat.visRelLevel] == visRelLevel & [dat.cueConflict] == cueConflict);
                
                % For multisensory trials, the stimulus is defined as the mean event rate
                % averaged between auditory and visual modalities.
                theseEventRates = 0.5*[dat(theseTrials).audEventRate] + 0.5*[dat(theseTrials).visEventRate];
                theseResponses = [dat(theseTrials).response];
                
                % Get psychometric fits.
                [choiceDataByRate,psychParams,psychCurve] = FitPsychometricCurve(theseResponses,theseEventRates);
                
                % Store psychometric fits in structure
                psychFits.trialTypes(trialTypeCount).name = trialTypeNames{trialTypeCount};
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).cueConflict = cueConflict;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).choiceDataByRate = choiceDataByRate;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).psychParams = psychParams;
                psychFits.trialTypes(trialTypeCount).conflictLevels(cueConflictCount).psychCurve = psychCurve;
               
                allConflictPSEEsts(cueConflictCount) = psychParams(5).est;
                allConflictPSEStdErrs(cueConflictCount) = psychParams(5).stdErr;
                
            end
            
            [visWeightEst,visWeightStdErr] = ComputeVisualWeight(cueConflicts,allConflictPSEEsts,allConflictPSEStdErrs);
            psychFits.trialTypes(trialTypeCount).visWeightEst = visWeightEst;
            psychFits.trialTypes(trialTypeCount).visWeightStdErr = visWeightStdErr;
            
            % Get single sensory threshold estimates corresponding to this
            % multisensory trial type in order to calculate the predicted visual weight.
            audThresholdEst = psychFits.trialTypes(audRelLevel).psychParams(2).est;
            audThresholdStdErr = psychFits.trialTypes(audRelLevel).psychParams(2).stdErr;
            visThresholdEst = psychFits.trialTypes(2+visRelLevel).psychParams(2).est;
            visThresholdStdErr = psychFits.trialTypes(2+visRelLevel).psychParams(2).stdErr;
            
            [predVisWeightEst,predVisWeightStdErr] = PredictVisualWeight(audThresholdEst,visThresholdEst,audThresholdStdErr,visThresholdStdErr);
            psychFits.trialTypes(trialTypeCount).predVisWeightEst = predVisWeightEst;
            psychFits.trialTypes(trialTypeCount).predVisWeightStdErr = predVisWeightStdErr;
            
        end
    end
    
    outPath = [dataFolder, thisDataset '_psychFits.mat'];
    delete(outPath);
    save(outPath,'psychFits');
    
end
