% Figure 2 generating script.
% John Sheppard, July 24, 2013
% Questions? sheppard@cshl.edu

close('all');

%% Fig. 2a

clear

% NOTE: THE LINE BELOW MUST INDICATE CORRECT PATH TO PSYCHOMETRIC DATA FITS.
% (See GetPsychometricFits.m and FitPsychometricCurve.m for details on
% computing the psychometric fits.)
load('../data/human1_psychFits.mat');

% RGB colors used for single sensory trial types.
audColorMap = [68 102 176; 21 102 57]/255;
visColorMap = [125 125 125; 0 0 0]/255;
colorMap = [audColorMap; visColorMap];

figure(1); clf(1); hold on;

categoryBoundary = 10.5;
plot(repmat(categoryBoundary,1,2),[0 1],'--k','linewidth',1);

for trialTypeCount = 1:4
    
    thisTrialTypeColor = colorMap(trialTypeCount,:);
    
    choiceDataByRate = psychFits.trialTypes(trialTypeCount).choiceDataByRate;
    psychCurve = psychFits.trialTypes(trialTypeCount).psychCurve;
    
    % Plot the fitted psychometric curve.
    plot(psychCurve(:,1),psychCurve(:,2),'-','Color',thisTrialTypeColor,'LineWidth',2);
    
    for eventRateCount = 1:size(choiceDataByRate,1);
        thisEventRate = choiceDataByRate(eventRateCount,1);
        numTrialsChoseHigh = choiceDataByRate(eventRateCount,2);
        numTrials = choiceDataByRate(eventRateCount,3);
        highRateChoicePropEst = numTrialsChoseHigh/numTrials;
        highRateChoicePropStdErr = sqrt(highRateChoicePropEst*(1-highRateChoicePropEst)/numTrials);
        highRateChoicePropConfInt = [highRateChoicePropEst-highRateChoicePropStdErr, highRateChoicePropEst+highRateChoicePropStdErr];
        plot(thisEventRate, highRateChoicePropEst,'o','Color',thisTrialTypeColor,'MarkerFaceColor',thisTrialTypeColor,'MarkerEdgeColor',thisTrialTypeColor);
        plot([thisEventRate,thisEventRate],highRateChoicePropConfInt,'-','Color',thisTrialTypeColor,'LineWidth',1.5);
    end
    
end

fontSize = 18;
set(gca,'tickdir','out','Box','off','ylim',[0 1],'ytick',0:.25:1,'xlim',[6.8 15.2],'xtick',7:15,'fontname','times','fontsize',fontSize);
title('Human','fontname','times','FontSize',fontSize);
xlabel('Event rate (events/second)','fontname','times','fontsize',fontSize);
ylabel('Proportion high choices','fontname','times','fontsize',fontSize);

%% Fig. 2b
% Single-sensory psychophysical thresholds for human subjects.

% Note: In the original published version of our manuscript, the color 
% conventions used in the human scatterplot (Fig. 2b) were reversed between
% modalities by mistake. That is, for Fig. 2b only, blue/green points
% indicated low-/high-reliability visual trials, and grey/black
% points indicated low/high-reliability auditory trials. The color
% conventions for this scatterplot have been corrected in this tutorial 
% (i.e., auditory = blue/green, visual = grey/black).

clear

% RGB colors used for single sensory trial types.
audColorMap = [68 102 176; 21 102 57]/255;
visColorMap = [125 125 125; 0 0 0]/255;
modalityColorMaps{1} = audColorMap;
modalityColorMaps{2} = visColorMap;

% Unique symbol identifiers assigned to each subject.
humanSymbols = {'p','o','d','^','v','>','<'};

trialTypeVarNames = {'audLowRel','audHighRel','visLowRel','visHighRel'};

for humanCount = 1:7
    
    % NOTE: THE LINE BELOW MUST INDICATE CORRECT PATH TO PSYCHOMETRIC DATA FITS.
    % (See GetPsychometricFits.m and FitPsychometricCurve.m for details on
    % computing the psychometric fits.)
    load(['../data/human' num2str(humanCount) '_psychFits.mat']);
    
    for trialTypeCount = 1:numel(trialTypeVarNames)
        
        thisTrialType = trialTypeVarNames{trialTypeCount};
        
        % Retrieve threshold (sigma) estimate and standard error from psychFits data structure.
        thresholdEsts(trialTypeCount,humanCount) = psychFits.trialTypes(trialTypeCount).psychParams(2).est;
        thresholdStdErrs(trialTypeCount,humanCount) = psychFits.trialTypes(trialTypeCount).psychParams(2).stdErr;
       
    end
end

% Now plot data.
figure(2); clf(2); hold on;

% Small jitters are applied to x-positions on plot to allow visualization of each
% symbol.
allJitters=[];
jitterCount=0;

for humanCount = 1:7
    thisSymbol = humanSymbols{humanCount};
    for modalityCount = 1:2
        thisColorMap = modalityColorMaps{modalityCount};
        for relCount = 1:2
            
            thisColor = thisColorMap(relCount,:);
            
            jitterCount=jitterCount+1;
            thisJitter = relCount + ((rand()-0.5)/3);
            allJitters = [allJitters, thisJitter];
            
            thisThresholdEst = thresholdEsts(((modalityCount-1)*2)+relCount,humanCount);
            thisThresholdStderr = thresholdStdErrs((modalityCount-1)*2+relCount,humanCount);
            thisThresholdCI = [thisThresholdEst - thisThresholdStderr, thisThresholdEst + thisThresholdStderr];
            
            thisModalityThresholdEsts(relCount) = thisThresholdEst;
            
            plot(repmat(thisJitter,[1,2]),thisThresholdCI,'color',thisColor,'linewidth',1);
            
        end
        
        plot(allJitters(jitterCount-1:jitterCount),thisModalityThresholdEsts,'--','color','k','linewidth',1);
        
    end  
end

% The symbol identifiers are plotted afterwards so that they are placed on top
% of the confidence intervals on plot.
jitterCount=0;
for humanCount = 1:7
    thisSymbol = humanSymbols{humanCount};
    for modalityCount = 1:2
        thisColorMap = modalityColorMaps{modalityCount};
        for relCount = 1:2
            
            thisColor = thisColorMap(relCount,:);
            
            jitterCount=jitterCount+1;
            thisJitter = allJitters(jitterCount);
            
            thisThresholdEst = thresholdEsts(((modalityCount-1)*2)+relCount,humanCount);
            
            if ~strcmp(thisSymbol,'p')
                plot(thisJitter,thisThresholdEst,thisSymbol,'MarkerSize',8,'MarkerFaceColor',thisColor,'MarkerEdgeColor',thisColor);
            else
                plot(thisJitter,thisThresholdEst,thisSymbol,'MarkerSize',12,'MarkerFaceColor',thisColor,'MarkerEdgeColor',thisColor);
            end
        end
    end
end

% Fix plot here.
xTickLabels = {'low rel.','high rel.'};
set(gca,'ylim',[-0.05 10],'xlim',[0.75 2.25],'XTick',1:2,'YTick',1:2:9,'XTickLabel',xTickLabels,'fontsize',20,'fontname','times');
set(gca,'TickDir','out','Box','off');
title('Humans','fontname','times','FontSize',20);
ylabel('Threshold (\sigma)','fontname','times','FontSize',20);
set(gcf, 'PaperUnits', 'inches','PaperSize', [10 10]);

%% Fig. 2c

clear

% NOTE: THE LINE BELOW MUST INDICATE CORRECT PATH TO BEHAVIORAL DATASETS.
load('../data/rat3_fig2_data_psychFits.mat');

% RGB colors used for single sensory trial types.
audColorMap = [68 102 176; 21 102 57]/255;
visColorMap = [0 0 0];
colorMap = [audColorMap; visColorMap];

figure(3); clf(3); hold on;

categoryBoundary = 12.5;
plot(repmat(categoryBoundary,1,2),[0 1],'--k','linewidth',1);

for trialTypeCount = 1:3

    thisTrialTypeColor = colorMap(trialTypeCount,:);
    choiceDataByRate = psychFits.trialTypes(trialTypeCount).choiceDataByRate;
    psychCurve = psychFits.trialTypes(trialTypeCount).psychCurve;
    
    % Plot the fitted psychometric curve.
    plot(psychCurve(:,1),psychCurve(:,2),'-','Color',thisTrialTypeColor,'LineWidth',2);
    
    for eventRateCount = 1:size(choiceDataByRate,1);
        thisEventRate = choiceDataByRate(eventRateCount,1);
        numTrialsChoseHigh = choiceDataByRate(eventRateCount,2);
        numTrials = choiceDataByRate(eventRateCount,3);
        highRateChoicePropEst = numTrialsChoseHigh/numTrials;
        highRateChoicePropStdErr = sqrt(highRateChoicePropEst*(1-highRateChoicePropEst)/numTrials);
        highRateChoicePropConfInt = [highRateChoicePropEst-highRateChoicePropStdErr, highRateChoicePropEst+highRateChoicePropStdErr];
        plot(thisEventRate, highRateChoicePropEst,'o','Color',thisTrialTypeColor,'MarkerFaceColor',thisTrialTypeColor,'MarkerEdgeColor',thisTrialTypeColor);
        plot([thisEventRate,thisEventRate],highRateChoicePropConfInt,'-','Color',thisTrialTypeColor,'LineWidth',1.5);
    end
    
end

fontSize = 18;
set(gca,'tickdir','out','Box','off','ylim',[0 1],'ytick',0:.25:1,'xlim',[8.8 16.2],'xtick',9:16,'fontname','times','fontsize',fontSize);
xlabel('Event rate (events/second)','fontname','times','fontsize',20);
ylabel('Proportion high choices','fontname','times','fontsize',20);
title('Rat','fontname','times','FontSize',20);

%% Fig. 2d
% Single-sensory psychophysical thresholds for rat subjects.

clear

% RGB colors used for single sensory trial types.
audColorMap = [68 102 176; 21 102 57]/255;
visColorMap = [0 0 0]/255;
modalityColorMaps{1} = audColorMap;
modalityColorMaps{2} = visColorMap;

% Unique symbol identifiers assigned to each subject.
ratSymbols = {'d','o','p','s','^'};

trialTypeVarNames = {'audLowRel','audHighRel','vis'};

numRelLevels = [2 1]; % 2 for auditory, 1 for visual

for ratCount = 1:5
    
    % NOTE: THE LINE BELOW MUST INDICATE CORRECT PATH TO PSYCHOMETRIC DATA FITS.
    % (See GetPsychometricFits.m and FitPsychometricCurve.m for details on
    % computing the psychometric fits.)
    load(['../data/rat' num2str(ratCount) '_psychFits.mat']);
    
    for trialTypeCount = 1:numel(trialTypeVarNames)
        
        thisTrialType = trialTypeVarNames{trialTypeCount};
        
        % Retrieve threshold (sigma) estimate and standard error from psychFits data structure.
        thresholdEsts(trialTypeCount,ratCount) = psychFits.trialTypes(trialTypeCount).psychParams(2).est;
        thresholdStdErrs(trialTypeCount,ratCount) = psychFits.trialTypes(trialTypeCount).psychParams(2).stdErr;
       
    end
end

% Now plot data.
figure(4); clf(4); hold on;

% Small jitters are applied to x-positions on plot to allow visualization of each
% symbol.
allJitters=[];
jitterCount=0;

for ratCount = 1:5
    thisSymbol = ratSymbols{ratCount};
    trialTypeCount = 0;
    for modalityCount = 1:2
        thisColorMap = modalityColorMaps{modalityCount};
        for relCount = 1:numRelLevels(modalityCount)
            trialTypeCount = trialTypeCount+1;
            thisColor = thisColorMap(relCount,:);
            
            jitterCount=jitterCount+1;
            thisJitter = trialTypeCount + ((rand()-0.5)/3);
            allJitters = [allJitters, thisJitter];
            
            thisThresholdEst = thresholdEsts(((modalityCount-1)*2)+relCount,ratCount);
            allThresholdEsts(trialTypeCount) = thisThresholdEst;
            thisThresholdStderr = thresholdStdErrs((modalityCount-1)*2+relCount,ratCount);
            thisThresholdCI = [thisThresholdEst - thisThresholdStderr, thisThresholdEst + thisThresholdStderr];
            
            plot(repmat(thisJitter,[1,2]),thisThresholdCI,'color',thisColor,'linewidth',2);
            
        end
        
        if modalityCount==1 && relCount == 2
            plot(allJitters(jitterCount-1:jitterCount),allThresholdEsts(1:2),'--','color','k','linewidth',1);
        elseif modalityCount==2
            plot(allJitters(jitterCount-1:jitterCount),allThresholdEsts(2:3),'--','color','k','linewidth',1);
        end
        
    end
end

% The symbol identifiers are plotted afterwards so that they are placed on
% top
% of the confidence intervals on plot.
jitterCount=0;
for ratCount = 1:5
    thisSymbol = ratSymbols{ratCount};
    trialTypeCount = 0;
    for modalityCount = 1:2
        thisColorMap = modalityColorMaps{modalityCount};
        for relCount = 1:numRelLevels(modalityCount)
            
            thisColor = thisColorMap(relCount,:);
            
            jitterCount=jitterCount+1;
            thisJitter = allJitters(jitterCount);
            
            thisThresholdEst = thresholdEsts(((modalityCount-1)*2)+relCount,ratCount);
            
            if ~strcmp(thisSymbol,'p')
                plot(thisJitter,thisThresholdEst,thisSymbol,'MarkerSize',8,'MarkerFaceColor',thisColor,'MarkerEdgeColor',thisColor);
            else
                plot(thisJitter,thisThresholdEst,thisSymbol,'MarkerSize',12,'MarkerFaceColor',thisColor,'MarkerEdgeColor',thisColor);
            end
            
        end
    end
end

% Fix plot here.
xTickLabels = {'low rel. aud','high rel. aud', 'vis'};
set(gca,'ylim',[-0.05 10],'xlim',[0.75 3.25],'XTick',1:3,'YTick',1:2:9,'XTickLabel',xTickLabels,'fontsize',20,'fontname','times');
set(gca,'TickDir','out','Box','off');
title('Rats','fontname','times','FontSize',20);
ylabel('Threshold (\sigma)','fontname','times','FontSize',20);
set(gcf, 'PaperUnits', 'inches','PaperSize', [10 10]);