function simulateEvidenceAccumulation(accumulation_time)
% function simulateEvidenceAccumulation(accumulation_time)
% 
% This function is adapted from code originally written by Carlos Brody
% to demonstrate reverse correlations for behavioral data on the Poisson
% events task.
%
% Some original code documentation by Carlos Brody:
% Here is a system that gets randomly timed events and makes a decision
% based on the total number of events.
%
% Inputs: accumulation_time -- Specify in milliseconds over what
% duration the accumulator will accumulate sensory evidence from the
% beginning of the 1000-ms stimulus.
% E.g. 1000 will acccumulate over entire stimulus (0-1000 ms).
% 500 will accumulate only over first half of stimulus (0-500 ms).
%
% This function plots the expected excess rate curve from such an
% accumulator in solid green, as well as a "shuffle control" curve in
% dashed green.
%
% Code modified/expanded by Anne Churchland (2012-2013).

if ~exist('accumulation_time','var')
    accumulation_time = 1000;
end

% generate all the trial event times and decisions

rates    = [4 6 8 9 10 12 14];  % events/sec
stimdur  = 1;                 	% secs

ntrials_per_rate = 2000;
binWidth = 0.02;              % generating binWidth
bins = binWidth/2:binWidth:stimdur;    % time axis for trial generation

% First column of following cell will be generating rate, second column
% will be event times, last column will be decision.
columns = {'rate' 'event_times' 'decision' 'total_events'};
alltrials = cell(numel(rates)*numel(ntrials_per_rate), 3);

trial_counter = 0;
num_events= [];
choices = [];

accumulation_time = accumulation_time/1000; % Convert this value from milliseconds to seconds.
boundary = 9*accumulation_time; % above this, decision= high-rate (+1); below, decision= low-rate (-1).

revcorr_binWidth = 0.025; % This is the sliding window size for computing the excess rate.
TimeVector   = revcorr_binWidth/2:revcorr_binWidth:stimdur;

for r = 1:numel(rates),
    for t = 1:ntrials_per_rate,
        % Make the event times and the decision for this trial
        
        event_times = bins(rand(1, numel(bins)) < rates(r)*binWidth);
        
        if      numel(event_times(find(event_times < accumulation_time))) > boundary, decision = 1;
        elseif  numel(event_times(find(event_times < accumulation_time))) < boundary, decision = -1;
        else
            if rand(1) > 0.5, decision = 1; else decision = -1;
            end;
        end;
        trial_counter = trial_counter + 1;
        
        alltrials{trial_counter,1} = rates(r);
        alltrials{trial_counter,2} = event_times;
        alltrials{trial_counter,3} = decision;
        alltrials{trial_counter,4} = numel(event_times);
        
        [num_events_each_bin times] = hist(event_times,TimeVector);
        num_events(trial_counter,:) = num_events_each_bin;
        choices(trial_counter) = decision;  % This is redundant with what is in alltrials but we duplicate it here for convenience.
        
    end;
end;

% Code below added by AKC

end_time = length(TimeVector);
thisColor = [0 100 0]/255;
target_event_count = 8; % The criterion number of events outside the sliding window
                        % used to select trials for computing the excess rate.

for i_time = 1:length(TimeVector)
    
    % We need to find all the trials where the number of events OUTSIDE the
    % window of interest is equal to the "target" event count.
    
    if i_time > 1
        num_events_outside_window = sum( [ sum(num_events(:,1:i_time-1),2)  sum(num_events(:,i_time+1:end_time),2)] , 2);
    else
        num_events_outside_window = ( [   sum(num_events(:,i_time+1:end_time),2)]) ;
    end;
    take_these  = find(num_events_outside_window== target_event_count);
    
    % Now, group the trials based on what the animal did and look at the
    % rates INSIDE the window of interest.
    num_events_inside_window = num_events(take_these,i_time);
    take_these_choices = choices(take_these);
    
    % These are the average event rates in the window for stimuli preceding left and
    % right choices.
    chose_left = num_events_inside_window(take_these_choices == -1);
    chose_right = num_events_inside_window(take_these_choices == 1);
    left_choice_mean_local_rates(i_time) = mean(chose_left)/revcorr_binWidth;
    right_choice_mean_local_rates(i_time) = mean(chose_right)/revcorr_binWidth;
    
    % This is the shuffle control.
    shuffle_choices = take_these_choices(randperm(length(take_these_choices)));
    chose_left = num_events_inside_window(shuffle_choices == -1);
    chose_right = num_events_inside_window(shuffle_choices == 1);
    % The mean rates for left-/right-choice "shuffle" trials should hover around the category boundary.
    left_choice_mean_local_rates_shuffle(i_time) = mean(chose_left)/revcorr_binWidth;
    right_choice_mean_local_rates_shuffle(i_time) = mean(chose_right)/revcorr_binWidth;
    
end

lpf_windowsize = 4; % this is the size of the filter for the moving average. 
% Keep in mind the bin width for the x-axis is set to 20 ms in this function.
CTA_est = right_choice_mean_local_rates - left_choice_mean_local_rates;
CTA_shuffle_est = right_choice_mean_local_rates_shuffle - left_choice_mean_local_rates_shuffle;

CTA_est_filter = moving_average(CTA_est,lpf_windowsize);

hold on;

plot(TimeVector, CTA_est_filter, '-','color',thisColor,'linewidth',1.2);
plot(TimeVector, CTA_shuffle_est, '--','color',thisColor,'linewidth',1.2);
set(gca,'tickdir','out','box','off','fontname','times','fontsize',20);

xlabel('Time preceding end of stimulus (ms)','fontsize',20);
ylabel('Excess rate (events/s)','fontsize',20);
% The timing on the x-axis was changed from 0-1 to 200-800 ms.
% In this toy example, the event rates did not exactly match the
% specific distributions used in rats or humans for the paper. The point
% of the simulations was simply to show how equal use of evidence across
% the trial yields a flat excess rate curve, whereas use of only evidence
% from part of the stimulus (e.g., first 1/2 of the stimulus) produces an
% excess rate curves that is elevated during only the corresponding portion
% of the stimulus duration.
set(gca,'xlim',[0 1],'xtick',0:.3333:1,'xticklabel',{'-800','-600','-400','-200'},'yticklabel','');

end
